@@ -284,14 +284,26 @@ ORBit_get_union_switch (CORBA_TypeCode  tc,
 	switch (tc->kind) {
 	case CORBA_tk_ulong:
 	case CORBA_tk_long:
-	case CORBA_tk_enum:
-		retval = *(CORBA_long *)*val;
-		if (update) *val = ((guchar *)*val) + sizeof (CORBA_long);
+	case CORBA_tk_enum: {
+		CORBA_long tmp;
+
+		memcpy (&tmp, *val, sizeof (CORBA_long));
+		retval = (glong) tmp;
+
+		if (update)
+			*val = ((guchar *)*val) + sizeof (CORBA_long);
+		}
 		break;
 	case CORBA_tk_ushort:
-	case CORBA_tk_short:
-		retval = *(CORBA_short *)*val;
-		if (update) *val = ((guchar *)*val) + sizeof (CORBA_short);
+	case CORBA_tk_short: {
+		CORBA_short tmp;
+
+		memcpy (&tmp, *val, sizeof (CORBA_short));
+		retval = (glong) tmp;
+
+		if (update)
+			*val = ((guchar *)*val) + sizeof (CORBA_short);
+		}
 		break;
 	case CORBA_tk_char:
 	case CORBA_tk_boolean:
@@ -939,10 +951,22 @@ ORBit_value_equivalent (gpointer *a, gpointer *b,
 		ALIGN_COMPARE (a, b, float, float,   ORBIT_ALIGNOF_CORBA_FLOAT);
 		ALIGN_COMPARE (a, b, double, double, ORBIT_ALIGNOF_CORBA_DOUBLE);
 
-		ALIGN_COMPARE (a, b, boolean, octet, ORBIT_ALIGNOF_CORBA_OCTET);
 		ALIGN_COMPARE (a, b, char,    octet, ORBIT_ALIGNOF_CORBA_OCTET);
 		ALIGN_COMPARE (a, b, octet,   octet, ORBIT_ALIGNOF_CORBA_OCTET);
 
+	case CORBA_tk_boolean: {
+		gboolean ba, bb;
+
+		*a = ALIGN_ADDRESS (*a, ORBIT_ALIGNOF_CORBA_OCTET);
+		*b = ALIGN_ADDRESS (*b, ORBIT_ALIGNOF_CORBA_OCTET);
+		ba = *(CORBA_octet *) *a;
+		bb = *(CORBA_octet *) *b;
+		*a = ((guchar *) *a) + sizeof (CORBA_octet);
+		*b = ((guchar *) *b) + sizeof (CORBA_octet);
+
+		return (ba && bb) || (!ba && !bb);
+	}
+
 	case CORBA_tk_string:
 		*a = ALIGN_ADDRESS (*a, ORBIT_ALIGNOF_CORBA_POINTER);
 		*b = ALIGN_ADDRESS (*b, ORBIT_ALIGNOF_CORBA_POINTER);
@@ -1021,15 +1045,19 @@ ORBit_value_equivalent (gpointer *a, gpointer *b,
 	}
 
 	case CORBA_tk_union: {
-		gint   union_align = tc->c_align;
-		gint   discrim_align = MAX (tc->discriminator->c_align, tc->c_align);
-		size_t union_size = ORBit_gather_alloc_info (tc);
+		gint     union_align = tc->c_align;
+		gint     discrim_align = MAX (tc->discriminator->c_align, tc->c_align);
+		size_t   union_size = ORBit_gather_alloc_info (tc);
+		gpointer a_orig, b_orig;
 
 		CORBA_TypeCode utc_a = ORBit_get_union_tag (
 			tc, (gconstpointer *)a, FALSE);
 		CORBA_TypeCode utc_b = ORBit_get_union_tag (
 			tc, (gconstpointer *)b, FALSE);
 
+		a_orig = *a;
+		b_orig = *b;
+
 		if (!CORBA_TypeCode_equal (utc_a, utc_b, ev))
 			return FALSE;
 
@@ -1045,8 +1073,8 @@ ORBit_value_equivalent (gpointer *a, gpointer *b,
 		if (!ORBit_value_equivalent (a, b, utc_a, ev))
 			return FALSE;
 
-		*a = ((guchar *) *a) + union_size;
-		*b = ((guchar *) *b) + union_size;
+		*a = ((guchar *) a_orig) + union_size;
+		*b = ((guchar *) b_orig) + union_size;
 		return TRUE;
 	}
 
